@@ -15,7 +15,6 @@ __all__ = (
 PY_35 = sys.version_info >= (3, 5)
 PY_352 = sys.version_info >= (3, 5, 2)
 
-EOF_MARKER = b''
 DEFAULT_LIMIT = 2 ** 16
 
 
@@ -41,7 +40,7 @@ if PY_35:
                 rv = yield from self.read_func()
             except EofStream:
                 raise StopAsyncIteration  # NOQA
-            if rv == EOF_MARKER:
+            if rv == b'':
                 raise StopAsyncIteration  # NOQA
             return rv
 
@@ -87,7 +86,7 @@ class StreamReader(AsyncStreamReaderMixin):
 
     total_bytes = 0
 
-    def __init__(self, limit=DEFAULT_LIMIT, timeout=None, loop=None):
+    def __init__(self, limit=DEFAULT_LIMIT, timer=None, loop=None):
         self._limit = limit
         if loop is None:
             loop = asyncio.get_event_loop()
@@ -97,10 +96,9 @@ class StreamReader(AsyncStreamReaderMixin):
         self._buffer_offset = 0
         self._eof = False
         self._waiter = None
-        self._canceller = None
         self._eof_waiter = None
         self._exception = None
-        self._timeout = timeout
+        self._timer = timer
 
     def __repr__(self):
         info = ['StreamReader']
@@ -128,11 +126,6 @@ class StreamReader(AsyncStreamReaderMixin):
             if not waiter.cancelled():
                 waiter.set_exception(exc)
 
-        canceller = self._canceller
-        if canceller is not None:
-            self._canceller = None
-            canceller.cancel()
-
     def feed_eof(self):
         self._eof = True
 
@@ -142,11 +135,6 @@ class StreamReader(AsyncStreamReaderMixin):
             if not waiter.cancelled():
                 waiter.set_result(True)
 
-        canceller = self._canceller
-        if canceller is not None:
-            self._canceller = None
-            canceller.cancel()
-
         waiter = self._eof_waiter
         if waiter is not None:
             self._eof_waiter = None
@@ -201,11 +189,6 @@ class StreamReader(AsyncStreamReaderMixin):
             if not waiter.cancelled():
                 waiter.set_result(False)
 
-        canceller = self._canceller
-        if canceller is not None:
-            self._canceller = None
-            canceller.cancel()
-
     @asyncio.coroutine
     def _wait(self, func_name):
         # StreamReader uses a future to link the protocol feed_data() method
@@ -215,18 +198,16 @@ class StreamReader(AsyncStreamReaderMixin):
         if self._waiter is not None:
             raise RuntimeError('%s() called while another coroutine is '
                                'already waiting for incoming data' % func_name)
+
         waiter = self._waiter = helpers.create_future(self._loop)
-        if self._timeout:
-            self._canceller = self._loop.call_later(self._timeout,
-                                                    self.set_exception,
-                                                    asyncio.TimeoutError())
         try:
-            yield from waiter
+            if self._timer:
+                with self._timer:
+                    yield from waiter
+            else:
+                yield from waiter
         finally:
             self._waiter = None
-            if self._canceller is not None:
-                self._canceller.cancel()
-                self._canceller = None
 
     @asyncio.coroutine
     def readline(self):
@@ -242,7 +223,7 @@ class StreamReader(AsyncStreamReaderMixin):
                 offset = self._buffer_offset
                 ichar = self._buffer[0].find(b'\n', offset) + 1
                 # Read from current offset to found b'\n' or to the end.
-                data = self._read_nowait(ichar - offset if ichar else -1)
+                data = self._read_nowait_chunk(ichar - offset if ichar else -1)
                 line.append(data)
                 line_size += len(data)
                 if ichar:
@@ -278,7 +259,7 @@ class StreamReader(AsyncStreamReaderMixin):
                         'might be infinite loop: \n%s', stack)
 
         if not n:
-            return EOF_MARKER
+            return b''
 
         if n < 0:
             # This used to just loop creating a new waiter hoping to
@@ -330,7 +311,6 @@ class StreamReader(AsyncStreamReaderMixin):
         #
         # I believe the most users don't know about the method and
         # they are not affected.
-        assert n is not None, "n should be -1"
         if self._exception is not None:
             raise self._exception
 
@@ -340,10 +320,7 @@ class StreamReader(AsyncStreamReaderMixin):
 
         return self._read_nowait(n)
 
-    def _read_nowait(self, n):
-        if not self._buffer:
-            return EOF_MARKER
-
+    def _read_nowait_chunk(self, n):
         first_buffer = self._buffer[0]
         offset = self._buffer_offset
         if n != -1 and len(first_buffer) - offset > n:
@@ -361,6 +338,19 @@ class StreamReader(AsyncStreamReaderMixin):
         self._buffer_size -= len(data)
         return data
 
+    def _read_nowait(self, n):
+        chunks = []
+
+        while self._buffer:
+            chunk = self._read_nowait_chunk(n)
+            chunks.append(chunk)
+            if n != -1:
+                n -= len(chunk)
+                if n == 0:
+                    break
+
+        return b''.join(chunks) if chunks else b''
+
 
 class EmptyStreamReader(AsyncStreamReaderMixin):
 
@@ -388,22 +378,22 @@ class EmptyStreamReader(AsyncStreamReaderMixin):
 
     @asyncio.coroutine
     def readline(self):
-        return EOF_MARKER
+        return b''
 
     @asyncio.coroutine
     def read(self, n=-1):
-        return EOF_MARKER
+        return b''
 
     @asyncio.coroutine
     def readany(self):
-        return EOF_MARKER
+        return b''
 
     @asyncio.coroutine
     def readexactly(self, n):
         raise asyncio.streams.IncompleteReadError(b'', n)
 
     def read_nowait(self):
-        return EOF_MARKER
+        return b''
 
 
 class DataQueue:
@@ -494,7 +484,7 @@ class ChunksQueue(DataQueue):
         try:
             return (yield from super().read())
         except EofStream:
-            return EOF_MARKER
+            return b''
 
     readany = read
 
@@ -525,6 +515,7 @@ class FlowControlStreamReader(StreamReader):
 
         self._stream = stream
         self._b_limit = limit * 2
+        self._allow_pause = False
 
         # resume transport reading
         if stream.paused:
@@ -534,6 +525,7 @@ class FlowControlStreamReader(StreamReader):
                 pass
             else:
                 self._stream.paused = False
+                self._allow_pause = True
 
     def _check_buffer_size(self):
         if self._stream.paused:
@@ -558,7 +550,7 @@ class FlowControlStreamReader(StreamReader):
 
         super().feed_data(data)
 
-        if (not self._stream.paused and
+        if (self._allow_pause and not self._stream.paused and
                 not has_waiter and self._buffer_size > self._b_limit):
             try:
                 self._stream.transport.pause_reading()
@@ -602,6 +594,7 @@ class FlowControlDataQueue(DataQueue):
 
         self._stream = stream
         self._limit = limit * 2
+        self._allow_pause = False
 
         # resume transport reading
         if stream.paused:
@@ -611,13 +604,14 @@ class FlowControlDataQueue(DataQueue):
                 pass
             else:
                 self._stream.paused = False
+                self._allow_pause = True
 
     def feed_data(self, data, size):
         has_waiter = self._waiter is not None and not self._waiter.cancelled()
 
         super().feed_data(data, size)
 
-        if (not self._stream.paused and
+        if (self._allow_pause and not self._stream.paused and
                 not has_waiter and self._size > self._limit):
             try:
                 self._stream.transport.pause_reading()
@@ -657,6 +651,6 @@ class FlowControlChunksQueue(FlowControlDataQueue):
         try:
             return (yield from super().read())
         except EofStream:
-            return EOF_MARKER
+            return b''
 
     readany = read
