@@ -12,6 +12,7 @@ class TestStreamReader(unittest.TestCase):
     DATA = b'line1\nline2\nline3\n'
 
     def setUp(self):
+        self.time_service = None
         self.loop = asyncio.new_event_loop()
         asyncio.set_event_loop(None)
 
@@ -19,6 +20,11 @@ class TestStreamReader(unittest.TestCase):
         self.loop.close()
 
     def _make_one(self, *args, **kwargs):
+        if 'timeout' in kwargs:
+            self.time_service = helpers.TimeService(self.loop, interval=0.01)
+            self.addCleanup(self.time_service.close)
+            kwargs['timer'] = self.time_service.timeout(kwargs.pop('timeout'))
+
         return streams.StreamReader(loop=self.loop, *args, **kwargs)
 
     def test_create_waiter(self):
@@ -159,7 +165,7 @@ class TestStreamReader(unittest.TestCase):
         self.assertEqual(b'', data)
 
         data = self.loop.run_until_complete(stream.read())
-        self.assertIs(data, streams.EOF_MARKER)
+        self.assertEqual(data, b'')
 
     @mock.patch('aiohttp.streams.internal_logger')
     def test_read_eof_infinit(self, internal_logger):
@@ -296,7 +302,6 @@ class TestStreamReader(unittest.TestCase):
 
         line = self.loop.run_until_complete(stream.readline())
         self.assertEqual(b'', line)
-        self.assertIs(line, streams.EOF_MARKER)
 
     def test_readline_read_byte_count(self):
         stream = self._make_one()
@@ -492,7 +497,6 @@ class TestStreamReader(unittest.TestCase):
         data = self.loop.run_until_complete(read_task)
 
         self.assertEqual(b'', data)
-        self.assertIs(data, streams.EOF_MARKER)
 
     def test_readany_exception(self):
         stream = self._make_one()
@@ -509,10 +513,8 @@ class TestStreamReader(unittest.TestCase):
         stream = self._make_one()
         stream.feed_data(b'line1\nline2\n')
 
-        self.assertEqual(
-            stream.read_nowait(), b'line1\nline2\n')
-        self.assertIs(
-            stream.read_nowait(), streams.EOF_MARKER)
+        self.assertEqual(stream.read_nowait(), b'line1\nline2\n')
+        self.assertEqual(stream.read_nowait(), b'')
         stream.feed_eof()
         data = self.loop.run_until_complete(stream.read())
         self.assertEqual(b'', data)
@@ -525,8 +527,7 @@ class TestStreamReader(unittest.TestCase):
             stream.read_nowait(4), b'line')
         self.assertEqual(
             stream.read_nowait(), b'1\nline2\n')
-        self.assertIs(
-            stream.read_nowait(), streams.EOF_MARKER)
+        self.assertEqual(stream.read_nowait(), b'')
         stream.feed_eof()
         data = self.loop.run_until_complete(stream.read())
         self.assertEqual(b'', data)
@@ -590,56 +591,6 @@ class TestStreamReader(unittest.TestCase):
         self.assertEqual(b'line1', data)
         self.assertTrue(stream.at_eof())
 
-    def test_set_exception_cancels_timeout(self):
-        stream = self._make_one(timeout=1)
-        task = helpers.ensure_future(stream.readany(), loop=self.loop)
-        self.loop.run_until_complete(asyncio.sleep(0, loop=self.loop))
-
-        self.assertIsNotNone(stream._canceller)
-        canceller = stream._canceller = mock.Mock()
-        stream.set_exception(ValueError())
-        self.assertIsNone(stream._canceller)
-        canceller.cancel.assert_called_with()
-        self.assertRaises(
-            ValueError, self.loop.run_until_complete, task)
-
-    def test_feed_eof_cancels_timeout(self):
-        stream = self._make_one(timeout=1)
-        task = helpers.ensure_future(stream.readany(), loop=self.loop)
-        self.loop.run_until_complete(asyncio.sleep(0, loop=self.loop))
-
-        self.assertIsNotNone(stream._canceller)
-        canceller = stream._canceller = mock.Mock()
-        stream.feed_eof()
-        self.assertIsNone(stream._canceller)
-        canceller.cancel.assert_called_with()
-        self.assertEqual(b'', self.loop.run_until_complete(task))
-
-    def test_feed_data_cancels_timeout(self):
-        stream = self._make_one(timeout=1)
-        task = helpers.ensure_future(stream.readany(), loop=self.loop)
-        self.loop.run_until_complete(asyncio.sleep(0, loop=self.loop))
-
-        self.assertIsNotNone(stream._canceller)
-        canceller = stream._canceller = mock.Mock()
-        stream.feed_data(b'data')
-        self.assertIsNone(stream._canceller)
-        canceller.cancel.assert_called_with()
-        self.assertEqual(b'data', self.loop.run_until_complete(task))
-
-    def test_wait_cancels_timeout(self):
-        # Read bytes.
-        stream = self._make_one(timeout=1)
-        task = helpers.ensure_future(stream._wait('test'), loop=self.loop)
-        self.loop.run_until_complete(asyncio.sleep(0, loop=self.loop))
-
-        self.assertIsNotNone(stream._canceller)
-        canceller = stream._canceller = mock.Mock()
-        stream._waiter.set_result(None)
-        self.loop.run_until_complete(task)
-        self.assertIsNone(stream._canceller)
-        canceller.cancel.assert_called_with()
-
 
 class TestEmptyStreamReader(unittest.TestCase):
 
@@ -659,16 +610,16 @@ class TestEmptyStreamReader(unittest.TestCase):
         self.assertTrue(s.at_eof())
         self.assertIsNone(
             self.loop.run_until_complete(s.wait_eof()))
-        self.assertIs(
-            self.loop.run_until_complete(s.read()), streams.EOF_MARKER)
-        self.assertIs(
-            self.loop.run_until_complete(s.readline()), streams.EOF_MARKER)
-        self.assertIs(
-            self.loop.run_until_complete(s.readany()), streams.EOF_MARKER)
+        self.assertEqual(
+            self.loop.run_until_complete(s.read()), b'')
+        self.assertEqual(
+            self.loop.run_until_complete(s.readline()), b'')
+        self.assertEqual(
+            self.loop.run_until_complete(s.readany()), b'')
         self.assertRaises(
             asyncio.IncompleteReadError,
             self.loop.run_until_complete, s.readexactly(10))
-        self.assertIs(s.read_nowait(), streams.EOF_MARKER)
+        self.assertEqual(s.read_nowait(), b'')
 
 
 class DataQueueMixin:
