@@ -378,6 +378,24 @@ def test_cannot_write_after_eof():
     assert not writer.write.called
 
 
+@asyncio.coroutine
+def test_repr_after_eof():
+    resp = StreamResponse()
+    writer = mock.Mock()
+    yield from resp.prepare(make_request('GET', '/', writer=writer))
+
+    assert resp.prepared
+    assert resp.started
+
+    resp.write(b'data')
+    writer.drain.return_value = ()
+    yield from resp.write_eof()
+    assert not resp.started
+    assert not resp.prepared
+    resp_repr = repr(resp)
+    assert resp_repr == '<StreamResponse OK eof>'
+
+
 @asyncio.coroutine
 def test_cannot_write_eof_before_headers():
     resp = StreamResponse()
